@@ -4,25 +4,20 @@ points.py
 
 Functions dealing with (n, d) points.
 """
+
 import copy
 
 import numpy as np
+from numpy import float64
 
-from .parent import Geometry3D
-from .geometry import plane_transform
+from . import caching, grouping, transformations, util
 from .constants import tol
+from .geometry import plane_transform
+from .parent import Geometry3D
 from .visual.color import VertexColor
 
 
-from . import util
-from . import caching
-from . import grouping
-from . import transformations
-
-
-def point_plane_distance(points,
-                         plane_normal,
-                         plane_origin=[0.0, 0.0, 0.0]):
+def point_plane_distance(points, plane_normal, plane_origin=None):
     """
     The minimum perpendicular distance of a point to a plane.
 
@@ -40,8 +35,11 @@ def point_plane_distance(points,
     distances : (n,) float
       Distance from point to plane
     """
-    points = np.asanyarray(points, dtype=np.float64)
-    w = points - plane_origin
+    points = np.asanyarray(points, dtype=float64)
+    if plane_origin is None:
+        w = points
+    else:
+        w = points - plane_origin
     distances = np.dot(plane_normal, w.T) / np.linalg.norm(plane_normal)
     return distances
 
@@ -85,7 +83,7 @@ def plane_fit(points):
       Unit normal vector of plane
     """
     # make sure input is numpy array
-    points = np.asanyarray(points, dtype=np.float64)
+    points = np.asanyarray(points, dtype=float64)
     assert points.ndim == 2 or points.ndim == 3
     # with only one point set, np.dot is faster
     if points.ndim == 2:
@@ -101,17 +99,14 @@ def plane_fit(points):
         # points offset by the plane origin
         x = points - C[:, None, :]
         # create a (p, 3, 3) matrix
-        M = np.einsum('pnd, pnm->pdm', x, x)
+        M = np.einsum("pnd, pnm->pdm", x, x)
     # run SVD
     N = np.linalg.svd(M)[0][..., -1]
     # return the centroid(s) and normal(s)
     return C, N
 
 
-def radial_sort(points,
-                origin,
-                normal,
-                start=None):
+def radial_sort(points, origin, normal, start=None):
     """
     Sorts a set of points radially (by angle) around an
     axis specified by origin and normal vector.
@@ -143,23 +138,24 @@ def radial_sort(points,
     else:
         normal, start = util.unitize([normal, start])
         if np.abs(1 - np.abs(np.dot(normal, start))) < tol.zero:
-            raise ValueError('start must not parallel with normal')
+            raise ValueError("start must not parallel with normal")
         axis0 = np.cross(start, normal)
         axis1 = np.cross(axis0, normal)
     vectors = points - origin
     # calculate the angles of the points on the axis
-    angles = np.arctan2(np.dot(vectors, axis0),
-                        np.dot(vectors, axis1))
+    angles = np.arctan2(np.dot(vectors, axis0), np.dot(vectors, axis1))
     # return the points sorted by angle
     return points[angles.argsort()[::-1]]
 
 
-def project_to_plane(points,
-                     plane_normal=[0, 0, 1],
-                     plane_origin=[0, 0, 0],
-                     transform=None,
-                     return_transform=False,
-                     return_planar=True):
+def project_to_plane(
+    points,
+    plane_normal,
+    plane_origin,
+    transform=None,
+    return_transform=False,
+    return_planar=True,
+):
     """
     Project (n, 3) points onto a plane.
 
@@ -180,13 +176,13 @@ def project_to_plane(points,
     """
 
     if np.all(np.abs(plane_normal) < tol.zero):
-        raise NameError('Normal must be nonzero!')
+        raise NameError("Normal must be nonzero!")
 
     if transform is None:
         transform = plane_transform(plane_origin, plane_normal)
 
     transformed = transformations.transform_points(points, transform)
-    transformed = transformed[:, 0:(3 - int(return_planar))]
+    transformed = transformed[:, 0 : (3 - int(return_planar))]
 
     if return_transform:
         polygon_to_3D = np.linalg.inv(transform)
@@ -217,7 +213,7 @@ def remove_close(points, radius):
 
     tree = cKDTree(points)
     # get the index of every pair of points closer than our radius
-    pairs = tree.query_pairs(radius, output_type='ndarray')
+    pairs = tree.query_pairs(radius, output_type="ndarray")
 
     # how often each vertex index appears in a pair
     # this is essentially a cheaply computed "vertex degree"
@@ -269,7 +265,7 @@ def k_means(points, k, **kwargs):
     from scipy.cluster.vq import kmeans
     from scipy.spatial import cKDTree
 
-    points = np.asanyarray(points, dtype=np.float64)
+    points = np.asanyarray(points, dtype=float64)
     points_std = points.std(axis=0)
     points_std[points_std < tol.zero] = 1
     whitened = points / points_std
@@ -312,10 +308,10 @@ def tsp(points, start=0):
       The euclidean distance between points in traversal
     """
     # points should be float
-    points = np.asanyarray(points, dtype=np.float64)
+    points = np.asanyarray(points, dtype=float64)
 
     if len(points.shape) != 2:
-        raise ValueError('points must be (n, dimension)!')
+        raise ValueError("points must be (n, dimension)!")
 
     # start should be an index
     start = int(start)
@@ -328,7 +324,7 @@ def tsp(points, start=0):
     traversal = np.zeros(len(points), dtype=np.int64) - 1
     traversal[0] = start
     # list of distances
-    distances = np.zeros(len(points) - 1, dtype=np.float64)
+    distances = np.zeros(len(points) - 1, dtype=float64)
     # a mask of indexes in order
     index_mask = np.arange(len(points), dtype=np.int64)
 
@@ -345,8 +341,7 @@ def tsp(points, start=0):
         # do NlogN distance query
         # use dot instead of .sum(axis=1) or np.linalg.norm
         # as it is faster, also don't square root here
-        dist = np.dot((points[unvisited] - current) ** 2,
-                      sum_ones)
+        dist = np.dot((points[unvisited] - current) ** 2, sum_ones)
 
         # minimum distance index
         min_index = dist.argmin()
@@ -379,20 +374,19 @@ def plot_points(points, show=True):
     import matplotlib.pyplot as plt
     from mpl_toolkits.mplot3d import Axes3D  # NOQA
 
-    points = np.asanyarray(points, dtype=np.float64)
+    points = np.asanyarray(points, dtype=float64)
 
     if len(points.shape) != 2:
-        raise ValueError('Points must be (n, 2|3)!')
+        raise ValueError("Points must be (n, 2|3)!")
 
     if points.shape[1] == 3:
         fig = plt.figure()
-        ax = fig.add_subplot(111, projection='3d')
+        ax = fig.add_subplot(111, projection="3d")
         ax.scatter(*points.T)
     elif points.shape[1] == 2:
         plt.scatter(*points.T)
     else:
-        raise ValueError('points not 2D/3D: {}'.format(
-            points.shape))
+        raise ValueError(f"points not 2D/3D: {points.shape}")
 
     if show:
         plt.show()
@@ -418,7 +412,7 @@ class PointCloud(Geometry3D):
           Metadata about points
         """
         self._data = caching.DataStore()
-        self._cache = caching.Cache(self._data.hash)
+        self._cache = caching.Cache(self._data.__hash__)
         self.metadata = {}
 
         if metadata is not None:
@@ -427,8 +421,8 @@ class PointCloud(Geometry3D):
         # load vertices
         self.vertices = vertices
 
-        if 'vertex_colors' in kwargs and colors is None:
-            colors = kwargs['vertex_colors']
+        if "vertex_colors" in kwargs and colors is None:
+            colors = kwargs["vertex_colors"]
 
         # save visual data to vertex color object
         self.visual = VertexColor(colors=colors, obj=self)
@@ -482,6 +476,10 @@ class PointCloud(Geometry3D):
 
         # copy vertex and face data
         copied._data.data = copy.deepcopy(self._data.data)
+
+        # copy visual data
+        copied.visual = copy.deepcopy(self.visual)
+
         # get metadata
         copied.metadata = copy.deepcopy(self.metadata)
 
@@ -523,8 +521,7 @@ class PointCloud(Geometry3D):
         self.vertices = self.vertices[unique]
 
         # apply unique mask to colors
-        if (self.colors is not None and
-                len(self.colors) == len(inverse)):
+        if self.colors is not None and len(self.colors) == len(inverse):
             self.colors = self.colors[unique]
 
     def apply_transform(self, transform):
@@ -537,8 +534,7 @@ class PointCloud(Geometry3D):
         transform : (4, 4) float
           Homogeneous transformation to apply to PointCloud
         """
-        self.vertices = transformations.transform_points(
-            self.vertices, matrix=transform)
+        self.vertices = transformations.transform_points(self.vertices, matrix=transform)
         return self
 
     @property
@@ -551,8 +547,7 @@ class PointCloud(Geometry3D):
         bounds : (2, 3) float
           Minimum, Maximum verteex
         """
-        return np.array([self.vertices.min(axis=0),
-                         self.vertices.max(axis=0)])
+        return np.array([self.vertices.min(axis=0), self.vertices.max(axis=0)])
 
     @property
     def extents(self):
@@ -588,18 +583,21 @@ class PointCloud(Geometry3D):
         vertices : (n, 3) float
           Points in the PointCloud
         """
-        return self._data['vertices']
+        return self._data.get("vertices", np.empty(shape=(0, 3), dtype=float64))
 
     @vertices.setter
-    def vertices(self, data):
-        if data is None:
-            self._data['vertices'] = None
-        else:
-            # we want to copy data for new object
-            data = np.array(data, dtype=np.float64, copy=True)
-            if not util.is_shape(data, (-1, 3)):
-                raise ValueError('Point clouds must be (n, 3)!')
-            self._data['vertices'] = data
+    def vertices(self, values):
+        """
+        Assign vertex values to the point cloud.
+
+        Parameters
+        --------------
+        values : (n, 3) float
+          Points in space
+        """
+        if values is None or len(values) == 0:
+            return self._data.data.pop("vertices", None)
+        self._data["vertices"] = np.asanyarray(values, order="C", dtype=float64)
 
     @property
     def colors(self):
@@ -630,6 +628,7 @@ class PointCloud(Geometry3D):
         """
 
         from scipy.spatial import cKDTree
+
         tree = cKDTree(self.vertices.view(np.ndarray))
         return tree
 
@@ -644,6 +643,7 @@ class PointCloud(Geometry3D):
           A watertight mesh of the hull of the points
         """
         from . import convex
+
         return convex.convex_hull(self.vertices)
 
     def scene(self):
@@ -656,6 +656,7 @@ class PointCloud(Geometry3D):
           Scene object containing this PointCloud
         """
         from .scene.scene import Scene
+
         return Scene(self)
 
     def show(self, **kwargs):
@@ -679,10 +680,8 @@ class PointCloud(Geometry3D):
           If file name is passed this is not required
         """
         from .exchange.export import export_mesh
-        return export_mesh(self,
-                           file_obj=file_obj,
-                           file_type=file_type,
-                           **kwargs)
+
+        return export_mesh(self, file_obj=file_obj, file_type=file_type, **kwargs)
 
     def query(self, input_points, **kwargs):
         """
@@ -697,6 +696,7 @@ class PointCloud(Geometry3D):
             Result of the query.
         """
         from .proximity import query_from_points
+
         return query_from_points(self.vertices, input_points, self.kdtree, **kwargs)
 
     def __add__(self, other):
@@ -705,10 +705,17 @@ class PointCloud(Geometry3D):
         else:
             # preserve colors
             # if one point cloud has no color property use black
-            other_colors = [[0, 0, 0, 255]] * \
-                len(other.vertices) if len(other.colors) == 0 else other.colors
-            self_colors = [[0, 0, 0, 255]] * \
-                len(self.vertices) if len(self.colors) == 0 else self.colors
+            other_colors = (
+                [[0, 0, 0, 255]] * len(other.vertices)
+                if len(other.colors) == 0
+                else other.colors
+            )
+            self_colors = (
+                [[0, 0, 0, 255]] * len(self.vertices)
+                if len(self.colors) == 0
+                else self.colors
+            )
             colors = np.vstack((self_colors, other_colors))
-        return PointCloud(vertices=np.vstack(
-            (self.vertices, other.vertices)), colors=colors)
+        return PointCloud(
+            vertices=np.vstack((self.vertices, other.vertices)), colors=colors
+        )
