@@ -7,17 +7,13 @@ Subclasses of Trimesh objects that are parameterized as primitives.
 Useful because you can move boxes and spheres around
 and then use trimesh operations on them at any point.
 """
-import numpy as np
+
 import abc
 
-from . import util
-from . import sample
-from . import caching
-from . import inertia
-from . import creation
-from . import triangles
-from . import transformations as tf
+import numpy as np
 
+from . import caching, creation, inertia, sample, triangles, util
+from . import transformations as tf
 from .base import Trimesh
 from .constants import log, tol
 
@@ -26,9 +22,9 @@ _IDENTITY = np.eye(4)
 _IDENTITY.flags.writeable = False
 
 
-class _Primitive(Trimesh):
+class Primitive(Trimesh):
     """
-    Geometric _Primitives which are a subclass of Trimesh.
+    Geometric Primitives which are a subclass of Trimesh.
     Mesh is generated lazily when vertices or faces are requested.
     """
 
@@ -38,7 +34,7 @@ class _Primitive(Trimesh):
 
     def __init__(self):
         # run the Trimesh constructor with no arguments
-        super(_Primitive, self).__init__()
+        super().__init__()
 
         # remove any data
         self._data.clear()
@@ -49,40 +45,39 @@ class _Primitive(Trimesh):
         self._cache.force_immutable = True
 
     def __repr__(self):
-        return '<trimesh.primitives.{}>'.format(
-            type(self).__name__)
+        return f"<trimesh.primitives.{type(self).__name__}>"
 
     @property
     def faces(self):
-        stored = self._cache['faces']
+        stored = self._cache["faces"]
         if util.is_shape(stored, (-1, 3)):
             return stored
         self._create_mesh()
-        return self._cache['faces']
+        return self._cache["faces"]
 
     @faces.setter
     def faces(self, values):
-        log.warning('primitive faces are immutable: not setting!')
+        log.warning("primitive faces are immutable: not setting!")
 
     @property
     def vertices(self):
-        stored = self._cache['vertices']
+        stored = self._cache["vertices"]
         if util.is_shape(stored, (-1, 3)):
             return stored
 
         self._create_mesh()
-        return self._cache['vertices']
+        return self._cache["vertices"]
 
     @vertices.setter
     def vertices(self, values):
         if values is not None:
-            log.warning('primitive vertices are immutable: not setting!')
+            log.warning("primitive vertices are immutable: not setting!")
 
     @property
     def face_normals(self):
         # we need to avoid the logic in the superclass that
         # is specific to the data model prioritizing faces
-        stored = self._cache['face_normals']
+        stored = self._cache["face_normals"]
         if util.is_shape(stored, (-1, 3)):
             return stored
         # just calculate if not stored
@@ -90,13 +85,13 @@ class _Primitive(Trimesh):
         normals = np.zeros((len(valid), 3))
         normals[valid] = unit
         # store and return
-        self._cache['face_normals'] = normals
+        self._cache["face_normals"] = normals
         return normals
 
     @face_normals.setter
     def face_normals(self, values):
         if values is not None:
-            log.warning('Primitive face normals are immutable!')
+            log.warning("Primitive face normals are immutable!")
 
     @property
     def transform(self):
@@ -129,7 +124,7 @@ class _Primitive(Trimesh):
         # get the constructor arguments
         kwargs.update(self.to_dict())
         # remove the type indicator, i.e. `Cylinder`
-        kwargs.pop('kind')
+        kwargs.pop("kind")
         # create a new object with kwargs
         primitive_copy = type(self)(**kwargs)
         # copy metadata
@@ -159,8 +154,9 @@ class _Primitive(Trimesh):
             vertices=self.vertices.copy(),
             faces=self.faces.copy(),
             face_normals=self.face_normals.copy(),
-            process=kwargs.pop('process', False),
-            **kwargs)
+            process=kwargs.pop("process", False),
+            **kwargs,
+        )
         return result
 
     def apply_transform(self, matrix):
@@ -176,10 +172,9 @@ class _Primitive(Trimesh):
         matrix: (4, 4) float
           Homogeneous transformation
         """
-        matrix = np.asanyarray(
-            matrix, order='C', dtype=np.float64)
+        matrix = np.asanyarray(matrix, order="C", dtype=np.float64)
         if matrix.shape != (4, 4):
-            raise ValueError('matrix must be `(4, 4)`!')
+            raise ValueError("matrix must be `(4, 4)`!")
         if util.allclose(matrix, _IDENTITY, 1e-8):
             # identity matrix is a no-op
             return self
@@ -195,19 +190,16 @@ class _Primitive(Trimesh):
         kinds = (Box, Cylinder, Capsule, Sphere)
         if isinstance(self, kinds) and abs(scale - 1.0) > 1e-8:
             # scale the primitive attributes
-            if hasattr(prim, 'height'):
+            if hasattr(prim, "height"):
                 prim.height *= scale
-            if hasattr(prim, 'radius'):
+            if hasattr(prim, "radius"):
                 prim.radius *= scale
-            if hasattr(prim, 'extents'):
+            if hasattr(prim, "extents"):
                 prim.extents *= scale
             # scale the translation of the current matrix
             current[:3, 3] *= scale
             # apply new matrix, rescale, translate, current
-            updated = util.multi_dot([
-                matrix,
-                tf.scale_matrix(1.0 / scale),
-                current])
+            updated = util.multi_dot([matrix, tf.scale_matrix(1.0 / scale), current])
         else:
             # without scaling just multiply
             updated = np.dot(matrix, current)
@@ -222,10 +214,10 @@ class _Primitive(Trimesh):
         return self
 
     def _create_mesh(self):
-        raise ValueError('Primitive doesn\'t define mesh creation!')
+        raise ValueError("Primitive doesn't define mesh creation!")
 
 
-class _PrimitiveAttributes(object):
+class PrimitiveAttributes:
     """
     Hold the mutable data which defines a primitive.
     """
@@ -236,7 +228,7 @@ class _PrimitiveAttributes(object):
 
         Parameters
         ------------
-        parent : _Primitive
+        parent : Primitive
           Parent object reference.
         defaults : dict
           The default values for this primitive type.
@@ -270,69 +262,59 @@ class _PrimitiveAttributes(object):
         # operation can be surprisingly slow and most
         # people never call it
         import pprint
+
         doc = (
-            'Store the attributes of a {name} object.\n\n' +
-            'When these values are changed, the mesh geometry will \n' +
-            'automatically be updated to reflect the new values.\n\n' +
-            'Available properties and their default values are:\n {defaults}' +
-            '\n\nExample\n---------------\n' +
-            'p = trimesh.primitives.{name}()\n' +
-            'p.primitive.radius = 10\n' +
-            '\n').format(
+            "Store the attributes of a {name} object.\n\n"
+            + "When these values are changed, the mesh geometry will \n"
+            + "automatically be updated to reflect the new values.\n\n"
+            + "Available properties and their default values are:\n {defaults}"
+            + "\n\nExample\n---------------\n"
+            + "p = trimesh.primitives.{name}()\n"
+            + "p.primitive.radius = 10\n"
+            + "\n"
+        ).format(
             name=self._parent.__class__.__name__,
-            defaults=pprint.pformat(
-                self._defaults,
-                width=-1)[1:-1])
+            defaults=pprint.pformat(self._defaults, width=-1)[1:-1],
+        )
         return doc
 
     def __getattr__(self, key):
-        if key.startswith('_'):
-            return super(_PrimitiveAttributes, self).__getattr__(key)
-        elif key == 'center':
+        if key.startswith("_"):
+            return super().__getattr__(key)
+        elif key == "center":
             # this whole __getattr__ is a little hacky
-            return self._data['transform'][:3, 3]
+            return self._data["transform"][:3, 3]
         elif key in self._defaults:
-            return util.convert_like(self._data[key],
-                                     self._defaults[key])
-        raise AttributeError(
-            "primitive object has no attribute '{}' ".format(key))
+            return util.convert_like(self._data[key], self._defaults[key])
+        raise AttributeError(f"primitive object has no attribute '{key}' ")
 
     def __setattr__(self, key, value):
-        if key.startswith('_'):
-            return super(_PrimitiveAttributes,
-                         self).__setattr__(key, value)
-        elif key == 'center':
+        if key.startswith("_"):
+            return super().__setattr__(key, value)
+        elif key == "center":
             value = np.array(value, dtype=np.float64)
             transform = np.eye(4)
             transform[:3, 3] = value
-            self._data['transform'] = transform
+            self._data["transform"] = transform
             return
         elif key in self._defaults:
             if self._mutable:
-                self._data[key] = util.convert_like(
-                    value, self._defaults[key])
+                self._data[key] = util.convert_like(value, self._defaults[key])
             else:
                 raise ValueError(
-                    'Primitive is configured as immutable! Cannot set attribute!')
+                    "Primitive is configured as immutable! Cannot set attribute!"
+                )
         else:
             keys = list(self._defaults.keys())
-            raise ValueError(
-                'Only default attributes {} can be set!'.format(keys))
+            raise ValueError(f"Only default attributes {keys} can be set!")
 
     def __dir__(self):
-        result = sorted(dir(type(self)) +
-                        list(self._defaults.keys()))
+        result = sorted(dir(type(self)) + list(self._defaults.keys()))
         return result
 
 
-class Cylinder(_Primitive):
-
-    def __init__(self,
-                 radius=1.0,
-                 height=1.0,
-                 transform=None,
-                 sections=32,
-                 mutable=True):
+class Cylinder(Primitive):
+    def __init__(self, radius=1.0, height=1.0, transform=None, sections=32, mutable=True):
         """
         Create a Cylinder Primitive, a subclass of Trimesh.
 
@@ -349,20 +331,20 @@ class Cylinder(_Primitive):
         mutable : bool
           Are extents and transform mutable after creation.
         """
-        super(Cylinder, self).__init__()
+        super().__init__()
 
-        defaults = {'height': 10.0,
-                    'radius': 1.0,
-                    'transform': np.eye(4),
-                    'sections': 32}
-        self.primitive = _PrimitiveAttributes(
+        defaults = {"height": 10.0, "radius": 1.0, "transform": np.eye(4), "sections": 32}
+        self.primitive = PrimitiveAttributes(
             self,
             defaults=defaults,
-            kwargs={'height': height,
-                    'radius': radius,
-                    'transform': transform,
-                    'sections': sections},
-            mutable=mutable)
+            kwargs={
+                "height": height,
+                "radius": radius,
+                "transform": transform,
+                "sections": sections,
+            },
+            mutable=mutable,
+        )
 
     @caching.cache_decorator
     def volume(self):
@@ -374,8 +356,7 @@ class Cylinder(_Primitive):
         volume : float
           Volume of the cylinder
         """
-        volume = ((np.pi * self.primitive.radius ** 2) *
-                  self.primitive.height)
+        volume = (np.pi * self.primitive.radius**2) * self.primitive.height
         return volume
 
     @caching.cache_decorator
@@ -393,7 +374,8 @@ class Cylinder(_Primitive):
             mass=self.volume,
             radius=self.primitive.radius,
             height=self.primitive.height,
-            transform=self.primitive.transform)
+            transform=self.primitive.transform,
+        )
         return tensor
 
     @caching.cache_decorator
@@ -423,9 +405,8 @@ class Cylinder(_Primitive):
         half = self.primitive.height / 2.0
         # apply the transform to the Z- aligned segment
         points = np.dot(
-            self.primitive.transform,
-            np.transpose([[0, 0, -half, 1],
-                          [0, 0, half, 1]])).T[:, :3]
+            self.primitive.transform, np.transpose([[0, 0, -half, 1], [0, 0, half, 1]])
+        ).T[:, :3]
         return points
 
     def to_dict(self):
@@ -439,10 +420,12 @@ class Cylinder(_Primitive):
         as_dict : dict
           Serializable data for this primitive.
         """
-        return {'kind': 'cylinder',
-                'transform': self.primitive.transform.tolist(),
-                'radius': float(self.primitive.radius),
-                'height': float(self.primitive.height), }
+        return {
+            "kind": "cylinder",
+            "transform": self.primitive.transform.tolist(),
+            "radius": float(self.primitive.radius),
+            "height": float(self.primitive.height),
+        }
 
     def buffer(self, distance):
         """
@@ -464,29 +447,28 @@ class Cylinder(_Primitive):
         buffered = Cylinder(
             height=self.primitive.height + distance * 2,
             radius=self.primitive.radius + distance,
-            transform=self.primitive.transform.copy())
+            transform=self.primitive.transform.copy(),
+        )
         return buffered
 
     def _create_mesh(self):
-        log.debug('creating mesh for Cylinder primitive')
-        mesh = creation.cylinder(radius=self.primitive.radius,
-                                 height=self.primitive.height,
-                                 sections=self.primitive.sections,
-                                 transform=self.primitive.transform)
+        log.debug("creating mesh for Cylinder primitive")
+        mesh = creation.cylinder(
+            radius=self.primitive.radius,
+            height=self.primitive.height,
+            sections=self.primitive.sections,
+            transform=self.primitive.transform,
+        )
 
-        self._cache['vertices'] = mesh.vertices
-        self._cache['faces'] = mesh.faces
-        self._cache['face_normals'] = mesh.face_normals
+        self._cache["vertices"] = mesh.vertices
+        self._cache["faces"] = mesh.faces
+        self._cache["face_normals"] = mesh.face_normals
 
 
-class Capsule(_Primitive):
-
-    def __init__(self,
-                 radius=1.0,
-                 height=10.0,
-                 transform=None,
-                 sections=32,
-                 mutable=True):
+class Capsule(Primitive):
+    def __init__(
+        self, radius=1.0, height=10.0, transform=None, sections=32, mutable=True
+    ):
         """
         Create a Capsule Primitive, a subclass of Trimesh.
 
@@ -503,20 +485,20 @@ class Capsule(_Primitive):
         mutable : bool
           Are extents and transform mutable after creation.
         """
-        super(Capsule, self).__init__()
+        super().__init__()
 
-        defaults = {'height': 1.0,
-                    'radius': 1.0,
-                    'transform': np.eye(4),
-                    'sections': 32}
-        self.primitive = _PrimitiveAttributes(
+        defaults = {"height": 1.0, "radius": 1.0, "transform": np.eye(4), "sections": 32}
+        self.primitive = PrimitiveAttributes(
             self,
             defaults=defaults,
-            kwargs={'height': height,
-                    'radius': radius,
-                    'transform': transform,
-                    'sections': sections},
-            mutable=mutable)
+            kwargs={
+                "height": height,
+                "radius": radius,
+                "transform": transform,
+                "sections": sections,
+            },
+            mutable=mutable,
+        )
 
     @property
     def transform(self):
@@ -533,10 +515,12 @@ class Capsule(_Primitive):
         as_dict : dict
           Serializable data for this primitive.
         """
-        return {'kind': 'capsule',
-                'transform': self.primitive.transform.tolist(),
-                'height': float(self.primitive.height),
-                'radius': float(self.primitive.radius)}
+        return {
+            "kind": "capsule",
+            "transform": self.primitive.transform.tolist(),
+            "height": float(self.primitive.height),
+            "radius": float(self.primitive.radius),
+        }
 
     @caching.cache_decorator
     def direction(self):
@@ -548,30 +532,26 @@ class Capsule(_Primitive):
         axis : (3,) float
           Vector along the cylinder axis
         """
-        axis = np.dot(self.primitive.transform,
-                      [0, 0, 1, 0])[:3]
+        axis = np.dot(self.primitive.transform, [0, 0, 1, 0])[:3]
         return axis
 
     def _create_mesh(self):
-        log.debug('creating mesh for `Capsule` primitive')
+        log.debug("creating mesh for `Capsule` primitive")
 
-        mesh = creation.capsule(radius=self.primitive.radius,
-                                height=self.primitive.height)
+        mesh = creation.capsule(
+            radius=self.primitive.radius, height=self.primitive.height
+        )
         mesh.apply_transform(self.primitive.transform)
 
-        self._cache['vertices'] = mesh.vertices
-        self._cache['faces'] = mesh.faces
-        self._cache['face_normals'] = mesh.face_normals
+        self._cache["vertices"] = mesh.vertices
+        self._cache["faces"] = mesh.faces
+        self._cache["face_normals"] = mesh.face_normals
 
 
-class Sphere(_Primitive):
-
-    def __init__(self,
-                 radius=1.0,
-                 center=None,
-                 transform=None,
-                 subdivisions=3,
-                 mutable=True):
+class Sphere(Primitive):
+    def __init__(
+        self, radius=1.0, center=None, transform=None, subdivisions=3, mutable=True
+    ):
         """
         Create a Sphere Primitive, a subclass of Trimesh.
 
@@ -589,28 +569,25 @@ class Sphere(_Primitive):
           Are extents and transform mutable after creation.
         """
 
-        super(Sphere, self).__init__()
+        super().__init__()
 
-        defaults = {'radius': 1.0,
-                    'transform': np.eye(4),
-                    'subdivisions': 3}
-        constructor = {'radius': float(radius),
-                       'subdivisions': int(subdivisions)}
+        defaults = {"radius": 1.0, "transform": np.eye(4), "subdivisions": 3}
+        constructor = {"radius": float(radius), "subdivisions": int(subdivisions)}
         # center is a helper method for "transform"
         # since a sphere is rotationally symmetric
         if center is not None:
             if transform is not None:
-                raise ValueError(
-                    'only one of `center` and `transform` may be passed!')
+                raise ValueError("only one of `center` and `transform` may be passed!")
             translate = np.eye(4)
             translate[:3, 3] = center
-            constructor['transform'] = translate
+            constructor["transform"] = translate
         elif transform is not None:
-            constructor['transform'] = transform
+            constructor["transform"] = transform
 
         # create the attributes object
-        self.primitive = _PrimitiveAttributes(
-            self, defaults=defaults, kwargs=constructor, mutable=mutable)
+        self.primitive = PrimitiveAttributes(
+            self, defaults=defaults, kwargs=constructor, mutable=mutable
+        )
 
     @property
     def center(self):
@@ -631,17 +608,23 @@ class Sphere(_Primitive):
         as_dict : dict
           Serializable data for this primitive.
         """
-        return {'kind': 'sphere',
-                'transform': self.primitive.transform.tolist(),
-                'radius': float(self.primitive.radius)}
+        return {
+            "kind": "sphere",
+            "transform": self.primitive.transform.tolist(),
+            "radius": float(self.primitive.radius),
+        }
 
     @property
     def bounds(self):
         # no docstring so will inherit Trimesh docstring
         # return exact bounds from primitive center and radius (rather than faces)
         # self.extents will also use this information
-        bounds = np.array([self.primitive.center - self.primitive.radius,
-                           self.primitive.center + self.primitive.radius])
+        bounds = np.array(
+            [
+                self.primitive.center - self.primitive.radius,
+                self.primitive.center + self.primitive.radius,
+            ]
+        )
         return bounds
 
     @property
@@ -662,7 +645,7 @@ class Sphere(_Primitive):
         area: float, surface area of the sphere Primitive
         """
 
-        area = 4.0 * np.pi * (self.primitive.radius ** 2)
+        area = 4.0 * np.pi * (self.primitive.radius**2)
         return area
 
     @caching.cache_decorator
@@ -675,7 +658,7 @@ class Sphere(_Primitive):
         volume: float, volume of the sphere Primitive
         """
 
-        volume = (4.0 * np.pi * (self.primitive.radius ** 3)) / 3.0
+        volume = (4.0 * np.pi * (self.primitive.radius**3)) / 3.0
         return volume
 
     @caching.cache_decorator
@@ -688,28 +671,22 @@ class Sphere(_Primitive):
         tensor: (3, 3) float
           3D inertia tensor.
         """
-        return inertia.sphere_inertia(
-            mass=self.volume,
-            radius=self.primitive.radius)
+        return inertia.sphere_inertia(mass=self.volume, radius=self.primitive.radius)
 
     def _create_mesh(self):
-        log.debug('creating mesh for Sphere primitive')
+        log.debug("creating mesh for Sphere primitive")
         unit = creation.icosphere(
-            subdivisions=self.primitive.subdivisions,
-            radius=self.primitive.radius)
+            subdivisions=self.primitive.subdivisions, radius=self.primitive.radius
+        )
 
         # apply the center offset here
-        self._cache['vertices'] = unit.vertices + self.primitive.center
-        self._cache['faces'] = unit.faces
-        self._cache['face_normals'] = unit.face_normals
+        self._cache["vertices"] = unit.vertices + self.primitive.center
+        self._cache["faces"] = unit.faces
+        self._cache["face_normals"] = unit.face_normals
 
 
-class Box(_Primitive):
-    def __init__(self,
-                 extents=None,
-                 transform=None,
-                 bounds=None,
-                 mutable=True):
+class Box(Primitive):
+    def __init__(self, extents=None, transform=None, bounds=None, mutable=True):
         """
         Create a Box Primitive as a subclass of Trimesh
 
@@ -725,30 +702,30 @@ class Box(_Primitive):
         mutable : bool
           Are extents and transform mutable after creation.
         """
-        super(Box, self).__init__()
-        defaults = {'transform': np.eye(4),
-                    'extents': np.ones(3)}
+        super().__init__()
+        defaults = {"transform": np.eye(4), "extents": np.ones(3)}
 
         if bounds is not None:
             # validate the multiple forms of input available here
             if extents is not None or transform is not None:
                 raise ValueError(
-                    'if `bounds` is passed `extents` and `transform` must not be!')
+                    "if `bounds` is passed `extents` and `transform` must not be!"
+                )
             bounds = np.array(bounds, dtype=np.float64)
             if bounds.shape != (2, 3):
-                raise ValueError('`bounds` must be (2, 3) float')
+                raise ValueError("`bounds` must be (2, 3) float")
             # create extents from AABB
             extents = bounds.ptp(axis=0)
             # translate to the center of the box
             transform = np.eye(4)
             transform[:3, 3] = bounds[0] + extents / 2.0
 
-        self.primitive = _PrimitiveAttributes(
+        self.primitive = PrimitiveAttributes(
             self,
             defaults=defaults,
-            kwargs={'extents': extents,
-                    'transform': transform},
-            mutable=mutable)
+            kwargs={"extents": extents, "transform": transform},
+            mutable=mutable,
+        )
 
     def to_dict(self):
         """
@@ -761,9 +738,11 @@ class Box(_Primitive):
         as_dict : dict
           Serializable data for this primitive.
         """
-        return {'kind': 'box',
-                'transform': self.primitive.transform.tolist(),
-                'extents': self.primitive.extents.tolist()}
+        return {
+            "kind": "box",
+            "transform": self.primitive.transform.tolist(),
+            "extents": self.primitive.extents.tolist(),
+        }
 
     @property
     def transform(self):
@@ -786,7 +765,8 @@ class Box(_Primitive):
         samples = sample.volume_rectangular(
             extents=self.primitive.extents,
             count=count,
-            transform=self.primitive.transform)
+            transform=self.primitive.transform,
+        )
         return samples
 
     def sample_grid(self, count=None, step=None):
@@ -808,23 +788,20 @@ class Box(_Primitive):
           Points inside the box
         """
 
-        if (count is not None and
-                step is not None):
-            raise ValueError('only step OR count can be specified!')
+        if count is not None and step is not None:
+            raise ValueError("only step OR count can be specified!")
 
         # create pre- transform bounds from extents
-        bounds = np.array([-self.primitive.extents,
-                           self.primitive.extents]) * .5
+        bounds = np.array([-self.primitive.extents, self.primitive.extents]) * 0.5
 
         if step is not None:
             grid = util.grid_arange(bounds, step=step)
         elif count is not None:
             grid = util.grid_linspace(bounds, count=count)
         else:
-            raise ValueError('either count or step must be specified!')
+            raise ValueError("either count or step must be specified!")
 
-        transformed = tf.transform_points(
-            grid, matrix=self.primitive.transform)
+        transformed = tf.transform_points(grid, matrix=self.primitive.transform)
         return transformed
 
     @property
@@ -833,8 +810,7 @@ class Box(_Primitive):
         Returns whether or not the current box is rotated at all.
         """
         if util.is_shape(self.primitive.transform, (4, 4)):
-            return not np.allclose(self.primitive.transform[
-                                   0:3, 0:3], np.eye(3))
+            return not np.allclose(self.primitive.transform[0:3, 0:3], np.eye(3))
         else:
             return False
 
@@ -852,14 +828,15 @@ class Box(_Primitive):
         return volume
 
     def _create_mesh(self):
-        log.debug('creating mesh for Box primitive')
-        box = creation.box(extents=self.primitive.extents,
-                           transform=self.primitive.transform)
+        log.debug("creating mesh for Box primitive")
+        box = creation.box(
+            extents=self.primitive.extents, transform=self.primitive.transform
+        )
 
         self._cache.cache.update(box._cache.cache)
-        self._cache['vertices'] = box.vertices
-        self._cache['faces'] = box.faces
-        self._cache['face_normals'] = box.face_normals
+        self._cache["vertices"] = box.vertices
+        self._cache["faces"] = box.faces
+        self._cache["face_normals"] = box.face_normals
 
     def as_outline(self):
         """
@@ -872,18 +849,15 @@ class Box(_Primitive):
         """
         # do the import in function to keep soft dependency
         from .path.creation import box_outline
+
         # return outline with same size as primitive
         return box_outline(
-            extents=self.primitive.extents,
-            transform=self.primitive.transform)
+            extents=self.primitive.extents, transform=self.primitive.transform
+        )
 
 
-class Extrusion(_Primitive):
-    def __init__(self,
-                 polygon=None,
-                 transform=None,
-                 height=1.0,
-                 mutable=True):
+class Extrusion(Primitive):
+    def __init__(self, polygon=None, transform=None, height=1.0, mutable=True):
         """
         Create an Extrusion primitive, which
         is a subclass of Trimesh.
@@ -903,19 +877,20 @@ class Extrusion(_Primitive):
         from shapely.geometry import Point
 
         # run the Trimesh init
-        super(Extrusion, self).__init__()
+        super().__init__()
         # set default values
-        defaults = {'polygon': Point([0, 0]).buffer(1.0),
-                    'transform': np.eye(4),
-                    'height': 1.0}
+        defaults = {
+            "polygon": Point([0, 0]).buffer(1.0),
+            "transform": np.eye(4),
+            "height": 1.0,
+        }
 
-        self.primitive = _PrimitiveAttributes(
+        self.primitive = PrimitiveAttributes(
             self,
             defaults=defaults,
-            kwargs={'transform': transform,
-                    'polygon': polygon,
-                    'height': height},
-            mutable=mutable)
+            kwargs={"transform": transform, "polygon": polygon, "height": height},
+            mutable=mutable,
+        )
 
     @caching.cache_decorator
     def area(self):
@@ -930,8 +905,7 @@ class Extrusion(_Primitive):
           Surface area of 3D extrusion
         """
         # area of the sides of the extrusion
-        area = abs(self.primitive.height *
-                   self.primitive.polygon.length)
+        area = abs(self.primitive.height * self.primitive.polygon.length)
         # area of the two caps of the extrusion
         area += self.primitive.polygon.area * 2
         return area
@@ -948,8 +922,7 @@ class Extrusion(_Primitive):
           Volume of 3D extrusion
         """
         # height may be negative
-        volume = abs(self.primitive.polygon.area *
-                     self.primitive.height)
+        volume = abs(self.primitive.polygon.area * self.primitive.height)
         return volume
 
     @caching.cache_decorator
@@ -965,8 +938,8 @@ class Extrusion(_Primitive):
         """
         # only consider rotation and signed height
         direction = np.dot(
-            self.primitive.transform[:3, :3],
-            [0.0, 0.0, np.sign(self.primitive.height)])
+            self.primitive.transform[:3, :3], [0.0, 0.0, np.sign(self.primitive.height)]
+        )
         return direction
 
     @property
@@ -991,9 +964,9 @@ class Extrusion(_Primitive):
         # no docstring for inheritance
         # calculate OBB using 2D polygon and known axis
         from . import bounds
+
         # find the 2D bounding box using the polygon
-        to_origin, box = bounds.oriented_bounds_2D(
-            self.primitive.polygon.exterior.coords)
+        to_origin, box = bounds.oriented_bounds_2D(self.primitive.polygon.exterior.coords)
         #  3D extents
         extents = np.append(box, abs(self.primitive.height))
         # calculate to_3D transform from 2D obb
@@ -1001,9 +974,7 @@ class Extrusion(_Primitive):
         rotation_Z[2, 3] = self.primitive.height / 2.0
         # combine the 2D OBB transformation with the 2D projection transform
         to_3D = np.dot(self.primitive.transform, rotation_Z)
-        obb = Box(transform=to_3D,
-                  extents=extents,
-                  mutable=False)
+        obb = Box(transform=to_3D, extents=extents, mutable=False)
         return obb
 
     def slide(self, distance):
@@ -1019,8 +990,7 @@ class Extrusion(_Primitive):
         distance = float(distance)
         translation = np.eye(4)
         translation[2, 3] = distance
-        new_transform = np.dot(self.primitive.transform.copy(),
-                               translation.copy())
+        new_transform = np.dot(self.primitive.transform.copy(), translation.copy())
         self.primitive.transform = new_transform
 
     def buffer(self, distance, distance_height=None, **kwargs):
@@ -1058,7 +1028,8 @@ class Extrusion(_Primitive):
             transform=self.primitive.transform.copy(),
             polygon=self.primitive.polygon.buffer(distance),
             height=height,
-            **kwargs)
+            **kwargs,
+        )
 
         # slide the stock along the axis
         buffered.slide(-np.sign(height) * distance_height)
@@ -1076,23 +1047,26 @@ class Extrusion(_Primitive):
         as_dict : dict
           Serializable data for this primitive.
         """
-        return {'kind': 'extrusion',
-                'polygon': self.primitive.polygon.wkt,
-                'transform': self.primitive.transform.tolist(),
-                'height': float(self.primitive.height)}
+        return {
+            "kind": "extrusion",
+            "polygon": self.primitive.polygon.wkt,
+            "transform": self.primitive.transform.tolist(),
+            "height": float(self.primitive.height),
+        }
 
     def _create_mesh(self):
-        log.debug('creating mesh for Extrusion primitive')
+        log.debug("creating mesh for Extrusion primitive")
         # extrude the polygon along Z
         mesh = creation.extrude_polygon(
             polygon=self.primitive.polygon,
             height=self.primitive.height,
-            transform=self.primitive.transform)
+            transform=self.primitive.transform,
+        )
 
         # check volume here in unit tests
         if tol.strict and mesh.volume < 0.0:
-            raise ValueError('matrix inverted mesh!')
+            raise ValueError("matrix inverted mesh!")
 
         # cache mesh geometry in the primitive
-        self._cache['vertices'] = mesh.vertices
-        self._cache['faces'] = mesh.faces
+        self._cache["vertices"] = mesh.vertices
+        self._cache["faces"] = mesh.faces
