@@ -6,11 +6,27 @@ import numpy as np
 from .. import caching, convex, grouping, inertia, transformations, units, util
 from ..exchange import export
 from ..parent import Geometry, Geometry3D
-from ..typed import Dict, List, NDArray, Optional, Tuple, Union, float64, int64
+from ..typed import (
+    ArrayLike,
+    Dict,
+    List,
+    NDArray,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
+    float64,
+    int64,
+)
 from ..util import unique_name
 from . import cameras, lighting
 from .transforms import SceneGraph
 
+# the types of objects we can create a scene from
+GeometryInput = Union[
+    Geometry, Sequence[Geometry], NDArray[Geometry], Dict[str, Geometry]
+]
+
 
 class Scene(Geometry3D):
     """
@@ -22,12 +38,12 @@ class Scene(Geometry3D):
 
     def __init__(
         self,
-        geometry: Union[Geometry, None, List[Geometry]] = None,
+        geometry: Optional[GeometryInput] = None,
         base_frame: str = "world",
         metadata: Optional[Dict] = None,
         graph: Optional[SceneGraph] = None,
         camera: Optional[cameras.Camera] = None,
-        lights: Optional[lighting.Light] = None,
+        lights: Optional[Sequence[lighting.Light]] = None,
         camera_transform: Optional[NDArray] = None,
     ):
         """
@@ -59,8 +75,9 @@ class Scene(Geometry3D):
         # create our cache
         self._cache = caching.Cache(id_function=self.__hash__)
 
-        # add passed geometry to scene
-        self.add_geometry(geometry)
+        if geometry is not None:
+            # add passed geometry to scene
+            self.add_geometry(geometry)
 
         # hold metadata about the scene
         self.metadata = {}
@@ -71,11 +88,12 @@ class Scene(Geometry3D):
             # if we've been passed a graph override the default
             self.graph = graph
 
-        self.camera = camera
-        self.lights = lights
-
-        if camera is not None and camera_transform is not None:
-            self.camera_transform = camera_transform
+        if lights is not None:
+            self.lights = lights
+        if camera is not None:
+            self.camera = camera
+            if camera_transform is not None:
+                self.camera_transform = camera_transform
 
     def apply_transform(self, transform):
         """
@@ -95,7 +113,7 @@ class Scene(Geometry3D):
 
     def add_geometry(
         self,
-        geometry: Union[Geometry, None, List[Geometry]] = None,
+        geometry: GeometryInput,
         node_name: Optional[str] = None,
         geom_name: Optional[str] = None,
         parent_node_name: Optional[str] = None,
@@ -211,7 +229,7 @@ class Scene(Geometry3D):
 
         return node_name
 
-    def delete_geometry(self, names: Union[set, str]) -> None:
+    def delete_geometry(self, names: Union[set, str, Sequence]) -> None:
         """
         Delete one more multiple geometries from the scene and also
         remove any node in the transform graph which references it.
@@ -242,7 +260,7 @@ class Scene(Geometry3D):
             if util.is_instance_named(geometry, "Trimesh"):
                 geometry.visual = ColorVisuals(mesh=geometry)
 
-    def __hash__(self) -> str:
+    def __hash__(self) -> int:
         """
         Return information about scene which is hashable.
 
@@ -636,7 +654,7 @@ class Scene(Geometry3D):
 
     def set_camera(
         self, angles=None, distance=None, center=None, resolution=None, fov=None
-    ) -> None:
+    ) -> cameras.Camera:
         """
         Create a camera object for self.camera, and add
         a transform to self.graph for it.
@@ -756,7 +774,7 @@ class Scene(Geometry3D):
         return self._camera
 
     @camera.setter
-    def camera(self, camera: cameras.Camera):
+    def camera(self, camera: Optional[cameras.Camera]):
         """
         Set a camera object for the Scene.
 
@@ -795,7 +813,7 @@ class Scene(Geometry3D):
         return self._lights
 
     @lights.setter
-    def lights(self, lights: List[lighting.Light]):
+    def lights(self, lights: Sequence[lighting.Light]):
         """
         Assign a list of light objects to the scene
 
@@ -829,7 +847,7 @@ class Scene(Geometry3D):
         )
         self.graph.base_frame = new_base
 
-    def dump(self, concatenate: bool = False) -> Union["Scene", List[Geometry]]:
+    def dump(self, concatenate: bool = False) -> Union[Geometry, List[Geometry]]:
         """
         Append all meshes in scene freezing transforms.
 
@@ -875,7 +893,7 @@ class Scene(Geometry3D):
             # if scene has mixed geometry this may drop some of it
             return util.concatenate(result)
 
-        return np.array(result)
+        return result
 
     def subscene(self, node: str) -> "Scene":
         """
@@ -918,8 +936,7 @@ class Scene(Geometry3D):
           Trimesh object which is a convex hull of all meshes in scene
         """
         points = util.vstack_empty([m.vertices for m in self.dump()])
-        hull = convex.convex_hull(points)
-        return hull
+        return convex.convex_hull(points)
 
     def export(self, file_obj=None, file_type=None, **kwargs):
         """
@@ -1077,7 +1094,7 @@ class Scene(Geometry3D):
             T_new[:3, 3] += offset
             self.graph[node_name] = T_new
 
-    def scaled(self, scale: Union[float, NDArray[float64]]) -> "Scene":
+    def scaled(self, scale: Union[float, ArrayLike]) -> "Scene":
         """
         Return a copy of the current scene, with meshes and scene
         transforms scaled to the requested factor.
